#include "stdafx.h"
#include "filters.h"

#include "Engine.h"

namespace Filters
{
	Compensate::Compensate(PClip _child, Constant::Neighbour _source, MotionClip _motion, Engines::Engine *_luma, Engines::Engine *_chroma, IScriptEnvironment *env)
	: GenericVideoFilter(_child), source(_source), motion(_motion), luma(_luma), chroma(_chroma),
	widthinblocks(vi.width/8), heightinblocks(vi.height/8) 
	{
		if (!vi.IsYV12())
			env->ThrowError("Compensate requires YV12.");
		//check that the motion clip is of the right size
		motion.checkmatch(vi);
	}

	PVideoFrame __stdcall Compensate::GetFrame(int n, IScriptEnvironment *env) 
	{
		int sourceframenumber = n + ((source == Constant::PREVIOUS) ? -1 : 1);
		if(sourceframenumber < 0 || sourceframenumber >= vi.num_frames)
			return child->GetFrame(n, env);

		const PVideoFrame sourceframe = child->GetFrame(sourceframenumber, env);
		const PMotionFrame motionframe = motion.GetFrame(n, env);
		PVideoFrame result = env->NewVideoFrame(vi);


		(*luma)(result,sourceframe,PLANAR_Y,motionframe);
		(*chroma)(result,sourceframe,PLANAR_U,motionframe, 2);
		(*chroma)(result,sourceframe,PLANAR_V,motionframe, 2);

		return result;
	}


	AVSValue __cdecl Create_SimpleCompensate(AVSValue args, void *user_data, IScriptEnvironment *env) 
	{
		return new Compensate(args[0].AsClip(), Constant::Neighbour(args[2].AsInt(Constant::PREVIOUS)), args[1].AsClip(), 
			new Engines::MoveEngine<HalfPixel>, new Engines::MoveEngine<HalfPixel>, env);  
	}

	AVSValue __cdecl Create_SimpleWarpCompensate(AVSValue args, void *user_data, IScriptEnvironment *env) 
	{
		bool chroma = args[3].AsBool(false);
		bool bilinear = args[4].AsBool(false);
		Engines::Engine *lumaengine = 0;
		//((bilinear) ? new SimpleWarpEngine<Bilinear,HalfPixel> : new SimpleWarpEngine<_Pixel,HalfPixel>);
		if(bilinear)
			lumaengine = new Engines::SimpleWarpEngine<Bilinear,HalfPixel>;
		else
			lumaengine = new Engines::SimpleWarpEngine<_Pixel,HalfPixel>;
		return new Compensate(args[0].AsClip(), Constant::Neighbour(args[2].AsInt(Constant::PREVIOUS)), args[1].AsClip(), 
			lumaengine, new Engines::MoveEngine<HalfPixel>, env);  
	}
}