#include "stdafx.h"
#include "filters.h"

namespace Filters
{
	class Canvas
	{
		int height, width;
		int pitch;
		unsigned char *pointer;

		void spot(int x, int y, unsigned char colour=128);
	public:
		Canvas(PVideoFrame &);
		void spot(XY position, unsigned char colour = 128) {spot(position.x,  position.y, colour);}
		void line(XY start, XY displacement);
	};

	Canvas::Canvas(PVideoFrame &frame)
	: width(frame->GetRowSize()), height(frame->GetHeight()), pitch(frame->GetPitch()), pointer(frame->GetWritePtr())
	{
	}

	void Canvas::spot(int x, int y, unsigned char colour)
	{
		if(x>=0 && x<width && y>=0 &&  y<height)
			pointer[ y*pitch + x ] = colour;
	}

	void Canvas::line(XY start, XY displacement)
	{
		if(displacement.x==0 && displacement.y==0)
		{
			spot(start, 235);
			return;
		}

		if(abs(displacement.x)>=abs(displacement.y))
		{
			double gradient = static_cast<double>(displacement.y)/displacement.x;
			if(displacement.x>0)
				for(int i = 0; i <= displacement.x; ++i)
					spot(start.x + i, start.y + static_cast<int>(gradient*i+0.5));
			else
				for(int i = 0; i <= -displacement.x; ++i)
					spot(start.x - i, start.y - static_cast<int>(gradient*i+0.5));		
		}
		else
		{
			double gradient = static_cast<double>(displacement.x)/displacement.y;
			if(displacement.y>0)
				for(int i = 0; i <= displacement.y; ++i)
					spot(start.x + static_cast<int>(gradient*i+0.5), start.y + i);
			else
				for(int i = 0; i <= -displacement.y; ++i)
					spot(start.x - static_cast<int>(gradient*i+0.5), start.y - i);		
		}
			spot(start, 235);

	}

	DrawMotion::DrawMotion(PClip _child, MotionClip _motion, IScriptEnvironment *env)
	: GenericVideoFilter(_child), motion(_motion), widthinblocks(vi.width/8), heightinblocks(vi.height/8) 
	{
		if (!vi.IsYV12())
			env->ThrowError("DrawMotion requires YV12.");
		//check that the motion clip is of the right size
		motion.checkmatch(vi);
	}


	PVideoFrame __stdcall DrawMotion::GetFrame(int n, IScriptEnvironment *env) 
	{


		PVideoFrame frame = child->GetFrame(n, env);
		env->MakeWritable(&frame);
		const PMotionFrame motionframe = motion.GetFrame(n, env);
		const int motionpitch = motionframe.GetPitch();

		Canvas canvas(frame);

		const CompressedMotionVector *motionrow = motionframe.GetReadPtr();
		for(int y = 0; y < heightinblocks; ++y)
		{
			for(int x = 0; x < widthinblocks; ++x)
			{
				MotionVector motionvector = MotionVector(motionrow[x]);
				canvas.line(XY (x*8+3,y*8+3),XY(motionvector.x/scale, motionvector .y/scale));
				//NB sign convention -- motion vectors point AWAY from the current frame
			}
			motionrow += motionpitch;
		}

		return frame;
	}



	AVSValue __cdecl Create_DrawMotion(AVSValue args, void *user_data, IScriptEnvironment *env) 
	{
		return new DrawMotion(args[0].AsClip(), args[1].AsClip(), env);  
	}
}