#pragma once
#include "Adapter.h"

namespace Engines
{
	using Adapters::Adapter;

	//template virtual methods are not supported
	//so we work around by having each virtual operator() use RTTI to call template apply
	class Engine
	{
	public:
		virtual void operator()(PVideoFrame &result, const PVideoFrame source, int plane, const PMotionFrame motion, int subsample = 1) const = 0;
		virtual void operator()(PVideoFrame &result, const PVideoFrame source, int plane, const PMotionFrame motion, const Adapter &adapter, int subsample = 1) const = 0;
		virtual string ID() const = 0;
	};

	template<class Interpolation>
	class MoveEngine : public Engine
	{
		template<class AdapterType, int subsample>
			void apply(PVideoFrame &result, const PVideoFrame source, int plane, const PMotionFrame motion,const AdapterType adapter) const;
	public:
		MoveEngine() {}
		virtual void operator()(PVideoFrame &result, const PVideoFrame source, int plane, const PMotionFrame motion, int subsample = 1) const;
		virtual void operator()(PVideoFrame &result, const PVideoFrame source, int plane, const PMotionFrame motion, const Adapter &adapter, int subsample = 1) const;
		string ID() const;// {return string("MoveEngine[") + Interpolation().ID() + string("]");}
	};

	/*
	template<class Interpolation, int subsample = 1>
	class SplitEngine : public Engine
	{};*/

	template<class WarpInterpolation, class CopyInterpolation>
	class SimpleWarpEngine : public Engine
	{
		template<class AdapterType, int subsample>
			void apply(PVideoFrame &result, const PVideoFrame source, int plane, const PMotionFrame motion, const AdapterType adapter) const;
	public:
		virtual void operator()(PVideoFrame &result, const PVideoFrame source, int plane, const PMotionFrame motion, int subsample = 1) const;
		virtual void operator()(PVideoFrame &result, const PVideoFrame source, int plane, const PMotionFrame motion, const Adapter &adapter, int subsample = 1) const;
		string ID() const;// {return string("SimpleWarpEngine[") + WarpInterpolation().ID() + string(", ") + CopyInterpolation().ID() + string("]");}
	};
}
	#include "MoveEngine_template.h"
	//#include "SimpleWarpEngine_template.h"
	#include "Engine_template.h"