#pragma once

namespace Engines
{

	//template virtual methods are not supported
	//so we work around by having each virtual operator() use RTTI to call template apply
	template<class Interpolation>
	inline void MoveEngine<Interpolation>::operator()
		(PVideoFrame &result, const PVideoFrame source, int plane, const PMotionFrame motion, int subsample) const
	{
		if(subsample==1)
			apply<Adapters::Direct, 1>(result, source, plane, motion, Adapters::Direct());
		else if(subsample==2)
			apply<Adapters::Direct, 2>(result, source, plane, motion, Adapters::Direct());
		else throw;
	}

	template<class WarpInterpolation, class CopyInterpolation>
	inline void SimpleWarpEngine<WarpInterpolation, CopyInterpolation>::operator()
		(PVideoFrame &result, const PVideoFrame source, int plane, const PMotionFrame motion, int subsample) const

	{
		if(subsample==1)
			apply<Adapters::Direct, 1>(result, source, plane, motion, Adapters::Direct());
		else if(subsample==2)
			apply<Adapters::Direct, 2>(result, source, plane, motion, Adapters::Direct());
		else throw;
	}

	template<class Interpolation>
	inline void MoveEngine<Interpolation>::operator()
		(PVideoFrame &result, const PVideoFrame source, int plane, const PMotionFrame motion, const Adapter &adapter, int subsample) const
	{
		if(const Adapters::Direct *direct = dynamic_cast<const Adapters::Direct *>(&adapter))
		{
			if(subsample==1)
				apply<Adapters::Direct, 1>(result, source, plane, motion, *direct);
			else if(subsample==2)
				apply<Adapters::Direct, 2>(result, source, plane, motion, *direct);
			else throw;
		}
		else if(const Adapters::ScaleVector *scalevector = dynamic_cast<const Adapters::ScaleVector *>(&adapter))
		{
			if(subsample==1)
				apply<Adapters::ScaleVector, 1>(result, source, plane, motion, *scalevector);
			else if(subsample==2)
				apply<Adapters::ScaleVector, 2>(result, source, plane, motion, *scalevector);
			else throw;
		}
	}

	template<class WarpInterpolation, class CopyInterpolation>
	inline void SimpleWarpEngine<WarpInterpolation, CopyInterpolation>::operator()
		(PVideoFrame &result, const PVideoFrame source, int plane, const PMotionFrame motion, const Adapter &adapter, int subsample) const
	{
		if(const Adapters::Direct *direct = dynamic_cast<const Adapters::Direct *>(&adapter))
		{
			if(subsample==1)
				apply<Adapters::Direct, 1>(result, source, plane, motion, *direct);
			else if(subsample==2)
				apply<Adapters::Direct, 2>(result, source, plane, motion, *direct);
			else throw;
		}
		else if(const Adapters::ScaleVector *scalevector = dynamic_cast<const Adapters::ScaleVector *>(&adapter))
		{
			if(subsample==1)
				apply<Adapters::ScaleVector, 1>(result, source, plane, motion, *scalevector);
			else if(subsample==2)
				apply<Adapters::ScaleVector, 2>(result, source, plane, motion, *scalevector);
			else throw;
		}
	}

	template<class Interpolation>
	string MoveEngine<Interpolation>::
		ID() const {return string("MoveEngine[") + Interpolation().ID() + string("]");}

	template<class WarpInterpolation, class CopyInterpolation>
	string SimpleWarpEngine<WarpInterpolation, CopyInterpolation>::
		ID() const {return string("SimpleWarpEngine[") + WarpInterpolation().ID() + string(", ") + CopyInterpolation().ID() + string("]");}

}