#include "stdafx.h"
#include "filters.h"

#include "Engine.h"
#include "WeightedAverage.h"

namespace Filters
{

	//ofstream timing("motion timing.txt");

	FPS::FPS(PClip _child, Constant::Neighbour _source, MotionClip _motion, double _newFPS, Engines::Engine *_luma, Engines::Engine *_chroma, IScriptEnvironment *env)
	: GenericVideoFilter(_child), source(_source), motion(_motion), ratio(_newFPS* double (vi.fps_denominator)/vi.fps_numerator), luma(_luma), chroma(_chroma)
	{
		//timing<<_newFPS  <<"\t"<<luma-> ID()  <<"\t"<<chroma-> ID()  <<"\n";totaltime = nocalls = 0;

		if (!vi.IsYV12())
			env->ThrowError("Motion FPS requires YV12.");
		//check that the motion clip is of the right size	
		motion.checkmatch(vi);

		double numerator = _newFPS;
		int denominator = 1;
		while(numerator < (1<<24) && denominator < (1<<24))
			numerator *= 2, denominator *= 2;
		vi.SetFPS(static_cast<int>(numerator+0.5), denominator);
		vi.num_frames = static_cast<int>(vi.num_frames*ratio)-1;
	}

	PVideoFrame __stdcall FPS::GetFrame(int n, IScriptEnvironment *env) 
	{
		int sourceframenumber = static_cast<int>(n/ratio);
		double fraction = n/ratio - sourceframenumber;
		if(fraction == 0)
			return child->GetFrame(sourceframenumber, env);

		if(source == Constant::NEXT)
		{
			++sourceframenumber;
			fraction = 1 - fraction;
		}
		int motionframenumber = sourceframenumber - ((source == Constant::PREVIOUS) ? -1 : 1);


		int scaledfraction = static_cast<int>(fraction*scale+0.5);
		if(sourceframenumber < 0/* || sourceframenumber >= vi.num_frames-1*/)
			return child->GetFrame(0, env);

		const PVideoFrame sourceframe = child->GetFrame(sourceframenumber, env);
		const PMotionFrame motionframe = motion.GetFrame(motionframenumber, env);
		PVideoFrame result = env->NewVideoFrame(vi);

		Adapters::ScaleVector scalevector(scaledfraction);

		//static __int64 totaltime = 0;
		//static int nocalls = 0;
		const int period = 100;
	//MeasureCycles cycles;
		(*luma)(result,sourceframe,PLANAR_Y,motionframe, scalevector);
	//if(nocalls> 0&&nocalls% period == 0) 
	//{timing <<totaltime/(period*100000) << "\tluma\t" <<luma-> ID ()<< "\n";
	//totaltime = 0;
	//timing.flush();}
	//++nocalls;
	//totaltime+= cycles.mark();
		(*chroma)(result,sourceframe,PLANAR_U,motionframe, scalevector, 2);
		(*chroma)(result,sourceframe,PLANAR_V,motionframe, scalevector, 2);

		return result;
	}

	AVSValue __cdecl Create_SimpleFPS(AVSValue args, void *user_data, IScriptEnvironment *env) 
	{
		return new FPS(args[0].AsClip(), Constant::Neighbour(args[3].AsInt(Constant::PREVIOUS)), args[2].AsClip(), args[1].AsFloat(), 
			new Engines::MoveEngine<HalfPixel>, new Engines::MoveEngine<HalfPixel>, env);  
	}

	AVSValue __cdecl Create_SimpleWarpFPS(AVSValue args, void *user_data, IScriptEnvironment *env) 
	{
		bool chroma = args[4].AsBool(false);
		bool bilinear = args[5].AsBool(false);
		Engines::Engine *lumaengine = 0;
		if(bilinear)
			lumaengine = new Engines::SimpleWarpEngine<Bilinear,HalfPixel>;
		else
			lumaengine = new Engines::SimpleWarpEngine<_Pixel,HalfPixel>;

		return new FPS(args[0].AsClip(), Constant::Neighbour(args[3].AsInt(Constant::PREVIOUS)), args[2].AsClip(), args[1].AsFloat(), 
			lumaengine, new Engines::MoveEngine<HalfPixel>, env);  
	}


	CombineFPS::CombineFPS(PClip _child, PClip _fromprevious, PClip _fromnext, double _newFPS, IScriptEnvironment *env)
	: GenericVideoFilter(_child), fromprevious(_fromprevious), fromnext(_fromnext), ratio(_newFPS*vi.fps_denominator/vi.fps_numerator)
	{
		double numerator = _newFPS;
		int denominator = 1;
		while(numerator < (1<<24) && denominator < (1<<24))
			numerator *= 2, denominator *= 2;
		vi.SetFPS(static_cast<int>(numerator+0.5), denominator);
		vi.num_frames = static_cast<int>(vi.num_frames*ratio)-1;
	}

	PVideoFrame __stdcall CombineFPS::GetFrame(int n, IScriptEnvironment *env) 
	{
		int sourceframenumber = static_cast<int>(n/ratio);
		double fraction = n/ratio - sourceframenumber;
		if(fraction == 0)
			return fromprevious->GetFrame(n, env);

		vector<WeightedFrame> data;
		data.push_back(WeightedFrame(fromprevious->GetFrame(n, env), 1-fraction));
		data.push_back(WeightedFrame(fromnext->GetFrame(n, env), fraction));

	return WeightedAverage(data, env, vi);
	}
	AVSValue __cdecl Create_CombineFPS(AVSValue args, void *user_data, IScriptEnvironment *env)
	{
		return new CombineFPS(args[0].AsClip(), args[2].AsClip(), args[3].AsClip(), args[1].AsFloat(), env);  
	};


	BlendFPS::BlendFPS(PClip _child, double _newFPS, double _aperture,IScriptEnvironment *env)
	: GenericVideoFilter(_child), ratio(_newFPS*vi.fps_denominator/vi.fps_numerator), aperture(_aperture),
	originalframecount(vi.num_frames)
	{
		double numerator = _newFPS;
		int denominator = 1;
		while(numerator < (1<<24) && denominator < (1<<24))
			numerator *= 2, denominator *= 2;
		vi.SetFPS(static_cast<int>(numerator+0.5), denominator);
		vi.num_frames = static_cast<int>(vi.num_frames*ratio)-1;
	}

	PVideoFrame __stdcall BlendFPS::GetFrame(int n, IScriptEnvironment *env) 
	{
		double midpoint = (n + 0.5)/ratio;
		double radius = (0.5/ratio)*aperture;
		double start = midpoint - radius;
		double end = midpoint + radius;
		if(start < 0)
			start = 0;
		if(end >= originalframecount)
			end = originalframecount - 0.01;

		double length = end - start;
		int startnumber = static_cast<int>(start);
		int endnumber = static_cast<int>(end);
		if(startnumber == endnumber)
			return child->GetFrame(startnumber, env);
	//compute the relative weights of the first, intermediate and last frames used
		//as startproportion : 1 : 1 :...: 1 : 1 : endproportion
		//note that these add up to length
		double startproportion = 1 - (start - startnumber);
		double endproportion = end - endnumber;

		vector<WeightedFrame> data;
		data.push_back(WeightedFrame(child->GetFrame(startnumber, env), startproportion/length));
		for (int i = startnumber + 1; i < endnumber; ++i)
			data.push_back(WeightedFrame(child->GetFrame(i, env), 1/length));
		data.push_back(WeightedFrame(child->GetFrame(endnumber, env), endproportion/length));

		return WeightedAverage(data, env, vi);
	}
	AVSValue __cdecl Create_BlendFPS(AVSValue args, void *user_data, IScriptEnvironment *env)
	{
		return new BlendFPS(args[0].AsClip(), args[1].AsFloat(), args[2].AsFloat(1.0), env);  
	};

}
