#include "stdafx.h"
#include "ConstantMotion.h"
#include "filters.h"
#include "Update_template.h"

namespace Filters
{

	//"c[from]i[initialise]i[iterate]i[subpixel]b[useassembly]b[reset]i", Create_FindMotion, 0);
	FindMotion::FindMotion(PClip _child, Constant::Neighbour _from, int _initialise, int _iterate, bool _subpixel, bool _useassembly, int _reset, IScriptEnvironment *env)
	: MotionFilter(_child), 
	from(_from),
	initialise(_initialise), iterate(_iterate), subpixel(_subpixel),
	useassembly(_useassembly), reset(_reset),
	lastcomputedframe(0), lastcomputednumber(-1)
	{
		if(  !(from == Constant::PREVIOUS 	|| from == Constant::NEXT) )
			throw AvisynthError("Motion: FindMotion requires direction = previous or direction = next.");

		//Colour space and size checks are in the parent class MotionFilter

		//SetCacheHints(CACHE_RANGE,3);
		//child->SetCacheHints(CACHE_RANGE,3);
	}
	//	ofstream output("calls.txt");
	//ofstream requestlog("requestlog.txt");


	//PVideoFrame __stdcall FindMotion::GetFrame(int n, IScriptEnvironment *env) 
	PMotionFrame FindMotion::GetMotionFrame(int n, IScriptEnvironment *env) 
	{
		//if(n%10==0)
		//for(map<int, int>::const_iterator i = calls.begin(); i != calls.end(); ++i)
		//	output<<i-> second << "\t" <<i-> first<< "\n";
		//requestlog  <<n<< "\n";requestlog.flush();

		if(n== lastcomputednumber)
			return lastcomputedframe;

		int fromframenumber = n + ((from == Constant::PREVIOUS) ? -1 : 1);

		if(fromframenumber < 0 || fromframenumber >= vi.num_frames)//return identically zero motion vectors
		{
			PMotionFrame result = Generation::ZeroMotion(env, vi);
			lastcomputedframe = result;
			lastcomputednumber = 0;
			return result;
		}
		//if we are at a reset point or there is no temporal data, initialise
		if(  (from == Constant::PREVIOUS) && n==1	||		(from == Constant::NEXT) && n==0	||	 reset!=0 && n%reset==1  )
		{
			const PVideoFrame current = child->GetFrame(n, env);
			const PVideoFrame fromframe = child->GetFrame(fromframenumber, env);

			PMotionFrame result = Generation::ZeroMotion(env, vi);
			for(int i = 0; i < initialise; ++i)
				if(subpixel)
					result = Generation::Update<HalfPixel>(current, fromframe, result, n+ i, env, vi, useassembly);
				else
					result = Generation::Update<_Pixel>(current, fromframe, result, n+ i, env, vi, useassembly);
				
			lastcomputedframe = result;
			lastcomputednumber = n;
			return result;
		}

		const PVideoFrame current = child->GetFrame(n, env);
		const PVideoFrame fromframe = child->GetFrame(fromframenumber, env);
		PMotionFrame result = GetFrame(n-1, env);//NB this is a direct call, not via cache
		for(int i = 0; i < iterate; ++i)
			if(subpixel)
				result = Generation::Update<HalfPixel>(current, fromframe, result, n+ i, env, vi, useassembly);
			else
				result = Generation::Update<_Pixel>(current, fromframe, result, n+ i, env, vi, useassembly);

		lastcomputedframe = result;
		lastcomputednumber = n;
		return lastcomputedframe;
	}



	AVSValue __cdecl Create_FindMotion(AVSValue args, void *user_data, IScriptEnvironment *env) 
	{
		return new FindMotion(args[0].AsClip(), Constant::Neighbour(args[1].AsInt(Constant::PREVIOUS)), args[2].AsInt(0), args[3].AsInt(1), args[4].AsBool(true), args[5].AsBool(true), args[6].AsInt(0), env);  
	}
}