/*
	Motion plugin for AviSynth

	Copyright (C) 2006 Mohan
  
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation.
	 
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
		
	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
	
	The author can be contacted at the doom9.org forum under mg262.
*/

#include "stdafx.h"
#include "filters.h"
#include "constant.h"
#include "Engine.h"

std::map<int, int> calls;

namespace ScriptFunction
{
	using namespace Constant;
	using namespace Filters;

	AVSValue __cdecl previous(AVSValue args, void *, IScriptEnvironment *)	{  return PREVIOUS;  }
	AVSValue __cdecl next(AVSValue args, void *, IScriptEnvironment *)		{  return NEXT;  }

	AVSValue __cdecl pixel(AVSValue args, void *, IScriptEnvironment *)		{  return PIXEL;  }
	AVSValue __cdecl halfpixel(AVSValue args, void *, IScriptEnvironment *)	{  return HALFPIXEL;  }
	AVSValue __cdecl bilinear(AVSValue args, void *, IScriptEnvironment *)		{  return BILINEAR;  }

	AVSValue _move(int accuracy) 
		{return Constant::move(static_cast<Accuracy>(accuracy));}
	AVSValue _warp(int warpaccuracy, int moveaccuracy) 
		{return Constant::warp(static_cast<Accuracy>(warpaccuracy), static_cast<Accuracy>(moveaccuracy));}

	AVSValue __cdecl move(AVSValue args, void *, IScriptEnvironment *)	{  return _move( args[0].AsInt(HALFPIXEL) );  }
	AVSValue __cdecl warp(AVSValue args, void *, IScriptEnvironment *)	{  return _warp( args[0].AsInt(BILINEAR), args[1].AsInt(HALFPIXEL) );  }
	AVSValue __cdecl warpfast(AVSValue args, void *, IScriptEnvironment *)	{  return _warp( PIXEL, HALFPIXEL );  }

	AVSValue __cdecl Create_MotionFPS(AVSValue args, void *, IScriptEnvironment *env)
	{
		if(!args[1].Defined())
			throw AvisynthError("Motion: MotionFPS requires a target frame rate");
		Constant::Neighbour source = Constant::Neighbour(  args[4].AsInt(Constant::PREVIOUS)  );

		MotionClip motion = (  args[5].Defined() ? args[5].AsClip() : 
			new FindMotion(args[0].AsClip(), source, 6, 1, true, true, 80, env)  );
		
		Engines::Engine *lumaengine		= (  args[2].Defined()  ? enginefromnumber(args[2].AsInt()) : new Engines::MoveEngine<HalfPixel>  );
		Engines::Engine *chromaengine	= (  args[3].Defined()  ? enginefromnumber(args[3].AsInt()) : lumaengine  );

		return new FPS(args[0].AsClip(), source, motion, args[1].AsFloat(), lumaengine, chromaengine, env);
	}
	AVSValue __cdecl Create_Compensate(AVSValue args, void *, IScriptEnvironment *env)
	{
		Constant::Neighbour source = Constant::Neighbour(  args[3].AsInt(Constant::PREVIOUS)  );

		MotionClip motion = (  args[4].Defined() ? args[4].AsClip() : 
			new FindMotion(args[0].AsClip(), source, 6, 1, true, true, 80, env)  );
		
		Engines::Engine *lumaengine		= (  args[1].Defined()  ? enginefromnumber(args[1].AsInt()) : new Engines::MoveEngine<HalfPixel>  );
		Engines::Engine *chromaengine	= (  args[2].Defined()  ? enginefromnumber(args[2].AsInt()) : lumaengine  );

		return new Compensate(args[0].AsClip(), source, motion, lumaengine, chromaengine, env);
	}
}


extern "C" __declspec(dllexport) const char *__stdcall AvisynthPluginInit2(IScriptEnvironment *env) 
{
	env->AddFunction("Previous", "", ScriptFunction::previous, 0);
	env->AddFunction("Next", "", ScriptFunction::next, 0);
	env->AddFunction("Pixel", "", ScriptFunction::pixel, 0);
	env->AddFunction("HalfPixel", "", ScriptFunction::halfpixel, 0);
	env->AddFunction("HPel", "", ScriptFunction::halfpixel, 0);
	env->AddFunction("Bilinear", "", ScriptFunction::bilinear, 0);
	env->AddFunction("Move", "[interpolation]i", ScriptFunction::move, 0);
	env->AddFunction("Warp", "[warpinterpolation]i[moveinterpolation]i", ScriptFunction::warp, 0);
	env->AddFunction("WarpFast", "", ScriptFunction::warpfast, 0);

	env->AddFunction("MotionFPS",		"c[FPS]f[method]i[chromamethod]i[source]i[motion]c", ScriptFunction::Create_MotionFPS, 0);
	env->AddFunction("Compensate",	        "c[method]i[chromamethod]i[source]i[motion]c", ScriptFunction::Create_Compensate, 0);

	env->AddFunction("SumStretchMask",	        "c[multiplier]i", Filters::Create_SumStretchMask, 0);
	env->AddFunction("SumStretchFPSMask",	        "c[FPS]f[multiplier]i[source]i", Filters::Create_SumStretchFPSMask, 0);

	env->AddFunction("FindMotion", "c[from]i[initialise]i[iterate]i[subpixel]b[useassembly]b[reset]i", Filters::Create_FindMotion, 0);
	env->AddFunction("UniformMotion", "cff", Filters::Create_UniformMotion, 0);
	env->AddFunction("DrawMotion", "cc", Filters::Create_DrawMotion, 0);
	env->AddFunction("SimpleCompensate",					"cc[source]i", Filters::Create_SimpleCompensate, 0);
	//env->AddFunction("ScaledCompensate",			"ccf[source]i", Filters::Create_ScaledCompensate, 0);
	env->AddFunction("SWC",								"cc[source]i[chroma]b[bilinear]b", Filters::Create_SimpleWarpCompensate, 0);
	env->AddFunction("SimpleWarpCompensate",	"cc[source]i[chroma]b[bilinear]b", Filters::Create_SimpleWarpCompensate, 0);
	env->AddFunction("SimpleFPS",						"cfc[source]i", Filters::Create_SimpleFPS, 0);
	env->AddFunction("SimpleWarpFPS",				"cfc[source]i[chroma]b[bilinear]b", Filters::Create_SimpleWarpFPS, 0);
	env->AddFunction("CombineFPS",					"cfcc", Filters::Create_CombineFPS, 0);
	env->AddFunction("BlendFPS",						"cf[aperture]f", Filters::Create_BlendFPS, 0);
	return "Motion plugin";
}
