#pragma once
//included in stdafx.h

const static int scalepower = 16;//must be even

const static int scale = 1<< scalepower;
//NB we can't use /scale, %scale because these are broken for negative numbers
inline int integralpart(int x) {return x>>scalepower;}
inline int semiscaledpart(int x) {return x>>(scalepower/2);}//used for multiplying two scaled numbers without overflow
inline int roundedpart(int x) {return (x+scale/2)>>scalepower;}
struct CompressedMotionVector;
struct Fractional//used for holding subpixel offsets
{
	unsigned int __x,__y;
	Fractional() {}
	//Fractional(unsigned int _x, unsigned int _y) : __x(_x&3), __y(_y&3) {}
	Fractional(int _x, int _y) : __x(_x&(scale-1)), __y(_y&(scale-1)) {}
	Fractional(const CompressedMotionVector *v) ;//optimisation
	unsigned int x() const {return __x;}
	unsigned int y() const {return __y;}
};

//______________________________________________________________________________

struct XY
{
	int x,y;
	XY(int _x, int _y) : x(_x), y(_y) {}
};

//______________________________________________________________________________
//NB sign convention -- motion vectors point AWAY from the current frame
//So to find target, take block coordinates in current frame and ADD motion vector.
//the size of CompressedMotionVector must divide 16 (so it can be packed into a VideoFrame properly)
struct MotionVector
{
	int x,y;
	MotionVector() {}
	MotionVector(int _x, int _y) : x(_x), y(_y) {}
	explicit MotionVector(CompressedMotionVector v);
	bool operator==(MotionVector v) const	{return x==v.x && y==v.y;}	
	bool operator!=(MotionVector v) const		{return x!=v.x || y!=v.y;}
};
struct CompressedMotionVector
{
	typedef int packedtype;
	packedtype x,y;
	CompressedMotionVector() {}
	//CompressedMotionVector(short _x, short _y) : x(_x), y(_y) {}
	explicit CompressedMotionVector(MotionVector v) : x(static_cast<packedtype>(v.x)), y(static_cast<packedtype>(v.y)){}
};
inline MotionVector::MotionVector(CompressedMotionVector v) : x(v.x), y(v.y) {}
//
//inline Fractional::Fractional(const CompressedMotionVector *v)
//{
//	int data = *reinterpret_cast<const int *>(v);
//	__x = (data)&3;
//	__y = (data>>16)&3;
//}

//classes defining interpolation accuracy, for use as template arguments
//trying to define class Pixel causes an internal compiler error...
class _Pixel
{
public:
	int round(int i) const {return (i + scale/2) & (~(scale - 1));}
	string ID() const {return "Pixel";}
};

class HalfPixel
{
public:
	int round(int i) const {return (i + scale/4) & (~(scale/2 - 1));}
	string ID() const {return "HalfPixel";}

};

class Bilinear
{
public:
	int round(int i) const {return i;}
	string ID() const {return "Bilinear";}
};