#pragma once
#include "Engine.h"

#include "copy88block_half.h"
#include "copy44block_half.h"

namespace Engines
{
	template<class Interpolation> 
		template<class AdapterType, int subsample>
			void MoveEngine<Interpolation>::apply
			(PVideoFrame &result, const PVideoFrame source, int plane, const PMotionFrame motion, const typename AdapterType adapter) const
	{
		const int blocksize = 8/subsample;
		const Interpolation interpolation;//may change this to pass it into the function in future
		const int width = source->GetRowSize()/subsample;
		const int height = source->GetHeight()/subsample;
		const int widthinblocks = width/blocksize;
		const int heightinblocks = height/blocksize;

		const unsigned char *sourcepointer = source->GetReadPtr(plane);
		const int sourcepitch = source->GetPitch(plane);
		const int resultpitch = result->GetPitch(plane);
		const int motionpitch = motion.GetPitch();

		const CompressedMotionVector *motionrow = motion.GetReadPtr();
		unsigned char *resultrow = result->GetWritePtr(plane);
		for(XY block(0, 0), pixel(0, 0); block.y < heightinblocks; ++block.y, pixel.y+=blocksize)
		{
			for(block.x = 0, pixel.x = 0; block.x < widthinblocks; ++block.x, pixel.x+=blocksize)
			{
				MotionVector motionvector = adapter.apply<subsample>(  MotionVector(motionrow[block.x])  );

			//int /*scaledsourcex = interpolation.round(  (block.x*8*scale+motionvector.x)/subsample  );
			//int */scaledsourcey = interpolation.round(  (block.y*8*scale+motionvector.y)/subsample  );
				int scaledsourcex = interpolation.round(  pixel.x*scale + motionvector.x  );
				int scaledsourcey = interpolation.round(  pixel.y*scale + motionvector.y  );

				int sourcex = integralpart(scaledsourcex);
				int sourcey = integralpart(scaledsourcey);
				const unsigned char *sourcecorner = sourcepointer + sourcey*sourcepitch + sourcex;
				Fractional fractional( scaledsourcex, scaledsourcey );

				//if(scaledsourcex<0||scaledsourcex> (width- blocksize)*scale)
				//	throw AvisynthError("scaledsourcex out of range");
				//if(scaledsourcey<0||scaledsourcey>  (height- blocksize)*scale||scaledsourcey>  (height- blocksize)*scale)
				//	throw AvisynthError("scaledsourcey out of range");

				//I think the compiler may misinterpret 8/subsample on occasions:
				Primitives::copyblock_P3<8/subsample, 8/subsample, Interpolation>
					( resultrow + pixel.x, resultpitch, sourcecorner, sourcepitch, fractional );

				//if(subsample==1)
				//	Primitives::copyblock_P3<8,8, Interpolation>( resultrow + pixel.x, resultpitch, sourcecorner, sourcepitch, fractional );
				//else 
				//	Primitives::copyblock_P3<4,4, Interpolation>( resultrow + pixel.x, resultpitch, sourcecorner, sourcepitch, fractional );
			}
			resultrow += blocksize*resultpitch;
			motionrow += motionpitch;
		}
		__asm emms;
	}

}
