#pragma once
#include "Engine.h"

#include "copy88block_half.h"
#include "copy44block_half.h"
#include "copy22block_half.h"
#include "copyblock_bilinear.h"
#include "warpblock.h"

namespace Engines
{

	template<class WarpInterpolation, class CopyInterpolation> 
		template<class AdapterType, int subsample>
			void SimpleWarpEngine<WarpInterpolation, CopyInterpolation>::apply
				(PVideoFrame &result, const PVideoFrame source, int plane, const PMotionFrame motion, const AdapterType adapter) const
	{
		const int blocksize = 8/subsample;
		const int halfblocksize = blocksize/2;
	//	const WarpInterpolation warpinterpolation;//may change this to pass it into the function in future
		const CopyInterpolation copyinterpolation;//may change this to pass it into the function in future

		const int width = source->GetRowSize()/subsample;
		const int height = source->GetHeight()/subsample;
		const int widthinblocks = width/blocksize;
		const int heightinblocks = height/blocksize;

		const unsigned char *sourcepointer = source->GetReadPtr(plane);
		const int sourcepitch = source->GetPitch(plane);
		const int resultpitch = result->GetPitch(plane);
		const int motionpitch = motion.GetPitch();

		//{const unsigned char *sourcerow = source->GetReadPtr(plane);
		//	unsigned char *resultrow = result->GetWritePtr(plane);
		//for(int y = 0; y < height; ++y, resultrow += resultpitch, sourcerow += sourcepitch)
		//	for(int x = 0; x < width; ++x)
		//		resultrow[x] = 128;}// sourcerow[x];}	
		//if(subsample==2) return;
		
		const CompressedMotionVector *motionrow = motion.GetReadPtr();
		unsigned char *resultrow = result->GetWritePtr(plane);
		const unsigned char *sourcerow = source->GetReadPtr(plane);

		XY block(0, 0);
		XY pixel(0, 0);
		//process first four lines
	//<AdapterType>width, pixely, resultrow, resultpitch, sourcepointer, sourcepitch, motionrow
	//rewrite all the border code in terms of
	//engine::processarea<AdapterType, referencewidth, referenceheight>//reference means non-subsampled
		//(resultposition, resultpitch, sourceposition, sourcepitch + 4 motion vectors)



		//PROCESS FIRST FOUR LINES
		{
			block.x = 0;
			pixel.x = 0;

			MotionVector topmotion = adapter.apply<subsample>(MotionVector(motionrow[block.x]));

			int scaledsourcex = copyinterpolation.round(  pixel.x*scale + topmotion.x  );
			int scaledsourcey = copyinterpolation.round(  pixel.y*scale + topmotion.y  );

			int othersourcex = integralpart(scaledsourcex);
			int othersourcey = integralpart(scaledsourcey);
			const unsigned char *sourcecorner = sourcepointer + othersourcey*sourcepitch + othersourcex;
			Fractional fractional( scaledsourcex, scaledsourcey );

			Primitives::copyblock_P3<4/subsample, 4/subsample, CopyInterpolation>
				( resultrow + pixel.x, resultpitch, sourcecorner, sourcepitch, fractional );
		}
		for(block.x = 0, pixel.x = halfblocksize; block.x < widthinblocks-1; ++block.x, pixel.x+=blocksize)
		{
			MotionVector leftmotion = MotionVector (motionrow[block.x]);
			//if the motion is uniform across the block, we can just copy it
			if(leftmotion == MotionVector(motionrow[block.x+1]) )
			{
				leftmotion = adapter.apply<subsample>(leftmotion);

				int scaledsourcex = copyinterpolation.round(  pixel.x*scale + leftmotion.x  );
				int scaledsourcey = copyinterpolation.round(  pixel.y*scale + leftmotion.y  );

				int othersourcex = integralpart(scaledsourcex);
				int othersourcey = integralpart(scaledsourcey);
				const unsigned char *sourcecorner = sourcepointer + othersourcey*sourcepitch + othersourcex;
				Fractional fractional( scaledsourcex, scaledsourcey );

				Primitives::copyblock_P3<4/subsample, 4/subsample, CopyInterpolation>
					( resultrow + pixel.x, resultpitch, sourcecorner, sourcepitch, fractional );
				Primitives::copyblock_P3<4/subsample, 4/subsample, CopyInterpolation>
					( resultrow + pixel.x+ halfblocksize, resultpitch, sourcecorner+ halfblocksize, sourcepitch, fractional );
			}
			//otherwise we have to interpolate the motion
			else
			{
				leftmotion							= adapter.apply<subsample>(leftmotion);
				MotionVector rightmotion		= adapter.apply<subsample>( MotionVector(motionrow[block.x+1]) );

				Primitives::warpblock<8/subsample, 4/subsample, WarpInterpolation>
					( resultrow + pixel.x, resultpitch, sourcerow + pixel.x, sourcepitch, 
					leftmotion, rightmotion, leftmotion,  rightmotion );
			}
		}
		{
			MotionVector topmotion = adapter.apply<subsample>(MotionVector(motionrow[block.x]));

			int scaledsourcex = copyinterpolation.round(  pixel.x*scale + topmotion.x  );
			int scaledsourcey = copyinterpolation.round(  pixel.y*scale + topmotion.y  );

			int othersourcex = integralpart(scaledsourcex);
			int othersourcey = integralpart(scaledsourcey);
			const unsigned char *sourcecorner = sourcepointer + othersourcey*sourcepitch + othersourcex;
			Fractional fractional( scaledsourcex, scaledsourcey );

			Primitives::copyblock_P3<4/subsample, 4/subsample, CopyInterpolation>
				( resultrow + pixel.x, resultpitch, sourcecorner, sourcepitch, fractional );
		}

		resultrow += halfblocksize*resultpitch;
		sourcerow += halfblocksize*sourcepitch;
		pixel.y += halfblocksize;




		//process main vertical section
		//PROCESS MAIN VERTICAL SECTION
		for(; block.y < heightinblocks-1; ++block.y, pixel.y+=blocksize)
		{
			block.x = 0;
			pixel.x = 0;

			//process first four columns
			MotionVector topmotion = MotionVector (motionrow[block.x]);
			//if the motion is uniform across the block, we can just copy it
			if(topmotion == MotionVector(motionrow[motionpitch +block.x]))
			{
				topmotion = adapter.apply<subsample>(topmotion);

				int scaledsourcex = copyinterpolation.round(  pixel.x*scale + topmotion.x  );
				int scaledsourcey = copyinterpolation.round(  pixel.y*scale + topmotion.y  );

				int othersourcex = integralpart(scaledsourcex);
				int othersourcey = integralpart(scaledsourcey);
				const unsigned char *sourcecorner = sourcepointer + othersourcey*sourcepitch + othersourcex;
				Fractional fractional( scaledsourcex, scaledsourcey );

				Primitives::copyblock_P3<4/subsample, 4/subsample, CopyInterpolation>
					( resultrow + pixel.x, resultpitch, sourcecorner, sourcepitch, fractional );
				Primitives::copyblock_P3<4/subsample, 4/subsample, CopyInterpolation>
					( resultrow + pixel.x+(4/subsample)*resultpitch, resultpitch, sourcecorner+(4/subsample)*sourcepitch, sourcepitch, fractional );
			}
			//otherwise we have to interpolate the motion
			else
			{
				topmotion							= adapter.apply<subsample>(topmotion);
				MotionVector bottommotion	= adapter.apply<subsample>( MotionVector(motionrow[motionpitch+ block.x]) );

				Primitives::warpblock<4/subsample, 8/subsample, WarpInterpolation>
					( resultrow + pixel.x, resultpitch, sourcerow + pixel.x, sourcepitch, 
					topmotion, topmotion, bottommotion,  bottommotion );
			}
			
			//main body of code
			for(block.x = 0, pixel.x = halfblocksize; block.x < widthinblocks-1; ++block.x, pixel.x+=blocksize)
			{
				MotionVector topleftmotion = MotionVector (motionrow[block.x]);
				//if the motion is uniform across the block, we can just copy it
				if(topleftmotion == MotionVector(motionrow[block.x+1]) &&
					topleftmotion == MotionVector(motionrow[motionpitch +block.x]) &&
					topleftmotion == MotionVector(motionrow[motionpitch +block.x+1]) )
				{
					topleftmotion = adapter.apply<subsample>(topleftmotion);

					int scaledsourcex = copyinterpolation.round(  pixel.x*scale + topleftmotion.x  );
					int scaledsourcey = copyinterpolation.round(  pixel.y*scale + topleftmotion.y  );

					int othersourcex = integralpart(scaledsourcex);
					int othersourcey = integralpart(scaledsourcey);
					const unsigned char *sourcecorner = sourcepointer + othersourcey*sourcepitch + othersourcex;
					Fractional fractional( scaledsourcex, scaledsourcey );

					Primitives::copyblock_P3<8/subsample, 8/subsample, CopyInterpolation>
						( resultrow + pixel.x, resultpitch, sourcecorner, sourcepitch, fractional );
				}
				//otherwise we have to interpolate the motion
				else
				{
					topleftmotion							= adapter.apply<subsample>(topleftmotion);
					MotionVector toprightmotion		= adapter.apply<subsample>( MotionVector(motionrow[block.x+1]) );
					MotionVector bottomleftmotion	= adapter.apply<subsample>( MotionVector(motionrow[motionpitch +block.x]) );
					MotionVector bottomrightmotion	= adapter.apply<subsample>( MotionVector(motionrow[motionpitch +block.x+1]) );

					Primitives::warpblock<8/subsample, 8/subsample, WarpInterpolation>
						( resultrow + pixel.x, resultpitch, sourcerow + pixel.x, sourcepitch, 
						topleftmotion, toprightmotion, bottomleftmotion,  bottomrightmotion );
				}
			}

			//process first four columns
			topmotion = MotionVector (motionrow[block.x]);
			//if the motion is uniform across the block, we can just copy it
			if(topmotion == MotionVector(motionrow[motionpitch +block.x]))
			{
				topmotion = adapter.apply<subsample>(topmotion);

				int scaledsourcex = copyinterpolation.round(  pixel.x*scale + topmotion.x  );
				int scaledsourcey = copyinterpolation.round(  pixel.y*scale + topmotion.y  );

				int othersourcex = integralpart(scaledsourcex);
				int othersourcey = integralpart(scaledsourcey);
				const unsigned char *sourcecorner = sourcepointer + othersourcey*sourcepitch + othersourcex;
				Fractional fractional( scaledsourcex, scaledsourcey );

				Primitives::copyblock_P3<4/subsample, 4/subsample, CopyInterpolation>
					( resultrow + pixel.x, resultpitch, sourcecorner, sourcepitch, fractional );
				Primitives::copyblock_P3<4/subsample, 4/subsample, CopyInterpolation>
					( resultrow + pixel.x+(4/subsample)*resultpitch, resultpitch, sourcecorner+(4/subsample)*sourcepitch, sourcepitch, fractional );
			}
			//otherwise we have to interpolate the motion
			else
			{
				topmotion							= adapter.apply<subsample>(topmotion);
				MotionVector bottommotion	= adapter.apply<subsample>( MotionVector(motionrow[motionpitch+ block.x]) );

				Primitives::warpblock<4/subsample, 8/subsample, WarpInterpolation>
					( resultrow + pixel.x, resultpitch, sourcerow + pixel.x, sourcepitch, 
					topmotion, topmotion, bottommotion,  bottommotion );
			}

			//update pointers for new vertical line
			resultrow +=blocksize*resultpitch;
			sourcerow += blocksize*sourcepitch;
			motionrow += motionpitch;
		}
		//process last four rows
		//***



		//PROCESS LAST FOUR ROWS
		{
			block.x = 0;
			pixel.x = 0;

			MotionVector topmotion = adapter.apply<subsample>(MotionVector(motionrow[block.x]));

			int scaledsourcex = copyinterpolation.round(  pixel.x*scale + topmotion.x  );
			int scaledsourcey = copyinterpolation.round(  pixel.y*scale + topmotion.y  );

			int othersourcex = integralpart(scaledsourcex);
			int othersourcey = integralpart(scaledsourcey);
			const unsigned char *sourcecorner = sourcepointer + othersourcey*sourcepitch + othersourcex;
			Fractional fractional( scaledsourcex, scaledsourcey );

			Primitives::copyblock_P3<4/subsample, 4/subsample, CopyInterpolation>
				( resultrow + pixel.x, resultpitch, sourcecorner, sourcepitch, fractional );
		}
		for(block.x = 0, pixel.x = halfblocksize; block.x < widthinblocks-1; ++block.x, pixel.x+=blocksize)
		{
			MotionVector leftmotion = MotionVector (motionrow[block.x]);
			//if the motion is uniform across the block, we can just copy it
			if(leftmotion == MotionVector(motionrow[block.x+1]) )
			{
				leftmotion = adapter.apply<subsample>(leftmotion);

				int scaledsourcex = copyinterpolation.round(  pixel.x*scale + leftmotion.x  );
				int scaledsourcey = copyinterpolation.round(  pixel.y*scale + leftmotion.y  );

				int othersourcex = integralpart(scaledsourcex);
				int othersourcey = integralpart(scaledsourcey);
				const unsigned char *sourcecorner = sourcepointer + othersourcey*sourcepitch + othersourcex;
				Fractional fractional( scaledsourcex, scaledsourcey );

				Primitives::copyblock_P3<4/subsample, 4/subsample, CopyInterpolation>
					( resultrow + pixel.x, resultpitch, sourcecorner, sourcepitch, fractional );
				Primitives::copyblock_P3<4/subsample, 4/subsample, CopyInterpolation>
					( resultrow + pixel.x+ halfblocksize, resultpitch, sourcecorner+ halfblocksize, sourcepitch, fractional );
			}
			//otherwise we have to interpolate the motion
			else
			{
				leftmotion							= adapter.apply<subsample>(leftmotion);
				MotionVector rightmotion		= adapter.apply<subsample>( MotionVector(motionrow[block.x+1]) );

				Primitives::warpblock<8/subsample, 4/subsample, WarpInterpolation>
					( resultrow + pixel.x, resultpitch, sourcerow + pixel.x, sourcepitch, 
					leftmotion, rightmotion, leftmotion,  rightmotion );
			}
		}
		{
			MotionVector topmotion = adapter.apply<subsample>(MotionVector(motionrow[block.x]));

			int scaledsourcex = copyinterpolation.round(  pixel.x*scale + topmotion.x  );
			int scaledsourcey = copyinterpolation.round(  pixel.y*scale + topmotion.y  );

			int othersourcex = integralpart(scaledsourcex);
			int othersourcey = integralpart(scaledsourcey);
			const unsigned char *sourcecorner = sourcepointer + othersourcey*sourcepitch + othersourcex;
			Fractional fractional( scaledsourcex, scaledsourcey );

			Primitives::copyblock_P3<4/subsample, 4/subsample, CopyInterpolation>
				( resultrow + pixel.x, resultpitch, sourcecorner, sourcepitch, fractional );
		}
		__asm emms;
	}

}