#include "stdafx.h"
#include "filters.h"
#include "SumStretch.h"

namespace Filters
{

	SumStretchMask::SumStretchMask(PClip _child, MotionClip _motion, int _multiplier, IScriptEnvironment *env)
		: GenericVideoFilter(_child), multiplier(_multiplier), motion(_motion)
	{
		vi.pixel_type = VideoInfo::CS_YV12;
		vi.width = motion.GetWidth();
		vi.height = motion.GetHeight();

		//until we can use Y8, the width and height need to be rounded up
		vi.width = ((vi.width + 3)/4)*4;
		vi.height = ((vi.height + 1)/2)*2;
	}

	PVideoFrame __stdcall SumStretchMask::GetFrame(int n, IScriptEnvironment *env)
	{
		const PMotionFrame motionframe = motion.GetFrame(n, env);
		PVideoFrame result = env->NewVideoFrame(vi);

		MaskMaker::SumStretchMaskMaker maskmaker(multiplier);
		maskmaker(result, motionframe);

		return result;
	}


	AVSValue __cdecl Create_SumStretchMask(AVSValue args, void *user_data, IScriptEnvironment *env) 
	{
		return new SumStretchMask(args[0].AsClip(), MotionClip(args[0].AsClip()), args[1].AsInt(), env);  


	}


	SumStretchFPSMask::SumStretchFPSMask(PClip _child, MotionClip _motion, double _newFPS, Constant::Neighbour _source, int _multiplier, IScriptEnvironment *env)
	: GenericVideoFilter(_child), motion(_motion), source(_source), ratio(_newFPS*vi.fps_denominator/vi.fps_numerator), multiplier(_multiplier)
	{
		vi.pixel_type = VideoInfo::CS_YV12;
		vi.width = motion.GetWidth();
		vi.height = motion.GetHeight();

		//until we can use Y8, the width and height need to be rounded up
		vi.width = ((vi.width + 3)/4)*4;
		vi.height = ((vi.height + 1)/2)*2;

		double numerator = _newFPS;
		int denominator = 1;
		while(numerator < (1<<24) && denominator < (1<<24))
			numerator *= 2, denominator *= 2;
		vi.SetFPS(static_cast<int>(numerator+0.5), denominator);
		vi.num_frames = static_cast<int>(vi.num_frames*ratio)-1;
	}

	PVideoFrame __stdcall SumStretchFPSMask::GetFrame(int n, IScriptEnvironment *env) 
	{
		int sourceframenumber = static_cast<int>(n/ratio);
		double fraction = n/ratio - sourceframenumber;
		if(fraction == 0)
		{
			PVideoFrame result = env->NewVideoFrame(vi);
			MaskMaker::ConstantMaskMaker maskmaker(0);
			maskmaker(result, motion.GetFrame(sourceframenumber, env));

			return result;
		}

		if(source == Constant::NEXT)
		{
			++sourceframenumber;
			fraction = 1 - fraction;
		}
		int motionframenumber = sourceframenumber - ((source == Constant::PREVIOUS) ? -1 : 1);


		int scaledfraction = static_cast<int>(fraction*scale+0.5);
		//if(sourceframenumber < 0 || sourceframenumber >= vi.num_frames-1/* ||
		//	motionframenumber < 0 || motionframenumber >= vi.num_frames-1*/)
		//	return child->GetFrame(0, env);

		//const PVideoFrame sourceframe = child->GetFrame(sourceframenumber, env);
		const PMotionFrame motionframe = motion.GetFrame(motionframenumber, env);
		PVideoFrame result = env->NewVideoFrame(vi);

		Adapters::ScaleVector scalevector(scaledfraction);


		MaskMaker::SumStretchMaskMaker maskmaker(multiplier);
		maskmaker(result, motionframe, scalevector);

		return result;
	}

	AVSValue __cdecl Create_SumStretchFPSMask(AVSValue args, void *user_data, IScriptEnvironment *env) 
	{
		return new SumStretchFPSMask(
			args[0].AsClip(), MotionClip(args[0].AsClip()),
			args[1].AsFloat(), Constant::Neighbour(args[3].AsInt(Constant::PREVIOUS)), 
			args[2].AsInt(), env);
	}

}