//#include "stdafx.h"
#include "Update.h"
#include "UpdateSet.h"

#include "search.h"
#include "search_pixel.h"
#include "search_halfpixel.h"

namespace Generation
{
	typedef Primitives::SearchCandidate Candidate;

	template<class Interpolation>
	inline bool Update<Interpolation>::isvalid(const XY position, MotionVector motionvector)
	{
		
	//needs updating to deal with fractional positions
		int scaledtargetx = (position.x + motionvector.x);
		if(scaledtargetx<0||scaledtargetx>/*=*/(width - 8)*scale) return false;

		int scaledtargety = (position.y + motionvector.y);
		return !(scaledtargety<0||scaledtargety>/*=*/(height - 8)*scale);
	}

	template<>
	inline void Update<_Pixel>::initialiseupdateset()
	{
		updateset.add(scale*1, 0);
		updateset.add(scale*2, 0); 
		updateset.add(0, -scale*1);
		updateset.add(0, -scale*2);
		updateset.add(-scale*1, 0);
		updateset.add(-scale*2, 0);
		updateset.add(0, scale*1);
		updateset.add(0, scale*2);
	}

	template<>
	inline void Update<HalfPixel>::initialiseupdateset()
	{
		// updateset.add(1, 0);
		updateset.add(scale/2, 0);
		updateset.add(scale*1, 0);
		updateset.add(scale*2, 0); 
		// updateset.add(0, -1);
		updateset.add(0, -scale/2);
		updateset.add(0, -scale*1);
		updateset.add(0, -scale*2);
		// updateset.add(-1, 0);
		updateset.add(-scale/2, 0);
		updateset.add(-scale*1, 0);
		updateset.add(-scale*2, 0);
		// updateset.add(0, 1);
		updateset.add(0, scale/2);
		updateset.add(0, scale*1);
		updateset.add(0, scale*2);
	}

	template<class Interpolation>
	Update<Interpolation>::Update(const PVideoFrame _current, const PVideoFrame _previous, const PMotionFrame _temporal, int seed, IScriptEnvironment *env, const VideoInfo &vi, const bool useassembly) 
	: width(_current->GetRowSize()), height(_current->GetHeight()),
	widthinblocks(_current->GetRowSize()/8), heightinblocks(_current->GetHeight()/8),
	current(_current), previous(_previous), temporal(_temporal), 
	//r//esult(env->NewVideoFrame(vi)),
	result(env, vi),
	currentpitch(_current->GetPitch()), previouspitch(_previous->GetPitch()), temporalpitch(_temporal.GetPitch()),
	resultpitch(result.GetPitch()),
	currentpointer(_current->GetReadPtr()), previouspointer(_previous->GetReadPtr()),
	updateset(seed)
	{
	initialiseupdateset();
	updateset.ready();

		const int updatepenalty =2*8*8;
		const int temporalpenalty =8*8/2;
		//const int updatepenalty =int (0.008*255*8*8+ 0.5);
		//const int temporalpenalty =int (0.002*255*8*8+ 0.5);

		//UpdateSet updateset(seed);


		CompressedMotionVector *resultrow = result.GetWritePtr();
		const CompressedMotionVector *temporalrow = temporal.GetReadPtr();
		for(XY block(0, 0), scaledblock(0,0); block.y < heightinblocks; ++block.y, scaledblock.y+=8*scale)
		{
	//IntegerUpdateSet updateset(seed+ block.y);

			//set up meandering
			bool headright = (block.y%2==0);
			int firstx = (headright ? 0 : widthinblocks-1);
			block.x = firstx;
			scaledblock.x = block.x*8*scale;
			int invalidx = (headright ? widthinblocks : -1 );
			int dx = (headright ? 1 : -1 );

			Candidate lastbestcandidate;
			for(; block.x != invalidx; block.x+=dx, scaledblock.x+=dx*8*scale)
			{
				//construct the list of candidates
				Candidate candidates[6];
				int nocandidates = 0;

				candidates[nocandidates++] =(Candidate(MotionVector(0,0), 8*8, 0, 0, 0));	//zero candidate
				//n.b. if zero candidate is disabled, need to use a check for all candidates being invalid

				MotionVector firstspatialcandidate(0, 0);
				MotionVector secondspatialcandidate(0, 0);

				if( block.x != firstx )//horizontal spatial candidate
				{
					firstspatialcandidate = lastbestcandidate.motionvector; //=resultrow[block.x-dx];
					//firstspatialcandidate = lastbestcandidate.motionvector; //=resultrow[block.x-dx];
					MotionVector updatecandidate = updateset.updateof(firstspatialcandidate);
					if(isvalid(scaledblock, firstspatialcandidate))	candidates[nocandidates++] =(Candidate(firstspatialcandidate, 0, previouspitch));
					if(isvalid(scaledblock, updatecandidate))		candidates[nocandidates++] =(Candidate(updatecandidate, updatepenalty, previouspitch));
				}
				if(block.y > 0)//vertical spatial candidate
				{
					//check for equality with other spatial candidates to speed up the search
					secondspatialcandidate = MotionVector(resultrow[block.x-resultpitch]);
					MotionVector updatecandidate = updateset.updateof(secondspatialcandidate);
					if( /* !(secondspatialcandidate.x == firstspatialcandidate.x && secondspatialcandidate.y == firstspatialcandidate.y)
						&&*/ isvalid(scaledblock, secondspatialcandidate)  )	
																				candidates[nocandidates++] =(Candidate(secondspatialcandidate, 0, previouspitch));
					if(isvalid(scaledblock, updatecandidate))		candidates[nocandidates++] =(Candidate(updatecandidate, updatepenalty, previouspitch));
				}
				if( block.x + 2 < widthinblocks && block.y + 2 < heightinblocks )//temporal candidate
				{
					//should check for equality with the spatial candidates to speed up the search
					MotionVector temporalcandidate = MotionVector(temporalrow[temporalpitch + temporalpitch + block.x + 2*dx]);
					if(	/*		!(temporalcandidate.x == firstspatialcandidate.x && temporalcandidate.y == firstspatialcandidate.y)
						&&		!(temporalcandidate.x == secondspatialcandidate.x && temporalcandidate.y == secondspatialcandidate.y) 
						&&*/ isvalid(scaledblock, temporalcandidate))	candidates[nocandidates++] =(Candidate(temporalcandidate, temporalpenalty, previouspitch));
				}
				if(nocandidates==0)
					candidates[nocandidates++] =(Candidate(MotionVector(0,0), 0, 0, 0, 0));

				//search through the candidates to find the best
				int bestcandidatenumber = -999999;//avoid compiler warning
			const unsigned char *const currentposition = currentpointer + integralpart(scaledblock.y) * currentpitch + integralpart(scaledblock.x);
			const unsigned char *const previousposition = previouspointer + integralpart(scaledblock.y) * previouspitch + integralpart(scaledblock.x);

					//if(useassembly)
				bestcandidatenumber = Primitives::search_P3<Interpolation>(currentposition, currentpitch, previousposition, previouspitch, candidates, nocandidates);
					//else
					//	bestcandidatenumber = search<Interpolation>(currentposition, currentpitch, previousposition, previouspitch, candidates, nocandidates);

				//WARNING: assembly functions do not call emms
		
				MotionVector bestmotionvector = candidates[bestcandidatenumber].motionvector;

				//if(abs(bestmotionvector.x)> 1000) throw;
				resultrow[block.x]= CompressedMotionVector(bestmotionvector);
				lastbestcandidate = candidates[bestcandidatenumber];

				//MotionVector recovered = MotionVector(	resultrow[block.x]);
				//if(recovered.x!= bestmotionvector.x||recovered.y!= bestmotionvector.y)
				//	throw;
			}
			resultrow += resultpitch;
			temporalrow += temporalpitch;
		}
		__asm emms;
		
	}

	template<class Interpolation>
	Update<Interpolation>::operator PMotionFrame()
	{
		return result;
	}

}

