#pragma once

namespace Engines
{
class Engine; 
}
namespace Constant
{
	enum Neighbour {PREVIOUS = -1, NEXT = 1};
}

namespace Filters
{
	//using Engines::Engines:: Engines::Engine;

	class FindMotion : public MotionFilter//GenericVideoFilter 
	{   
		Constant::Neighbour from;
		int initialise, iterate;
		bool subpixel;
		bool useassembly;//temporary
		int reset;//temporary


		PMotionFrame lastcomputedframe;
		int lastcomputednumber;
	public:
		FindMotion(PClip _child, Constant::Neighbour _from, int _initialise, int _iterate, bool _subpixel, bool _useassembly, int _reset, IScriptEnvironment *env);
	//	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment *env);
		virtual PMotionFrame GetMotionFrame(int n, IScriptEnvironment *env); 
	};
	AVSValue __cdecl Create_FindMotion(AVSValue args, void *user_data, IScriptEnvironment *env);


	class UniformMotion : public MotionFilter//GenericVideoFilter 
	{   
		MotionVector constantmotionvector;
	public:
		UniformMotion(PClip _child, MotionVector _constantmotionvector, IScriptEnvironment *env);
		//PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment *env);
		virtual PMotionFrame GetMotionFrame(int n, IScriptEnvironment *env); 
	};
	AVSValue __cdecl Create_UniformMotion(AVSValue args, void *user_data, IScriptEnvironment *env);



	class DrawMotion : public GenericVideoFilter 
	{   
		int widthinblocks, heightinblocks;
		MotionClip motion;
	public:
		DrawMotion(PClip _child, MotionClip _motion, IScriptEnvironment *env);
		PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment *env);
	};
	AVSValue __cdecl Create_DrawMotion(AVSValue args, void *user_data, IScriptEnvironment *env);


	class Compensate : public GenericVideoFilter 
	{   
		int widthinblocks, heightinblocks;
		Constant::Neighbour source;
		MotionClip motion;
		Engines::Engine *luma, *chroma;
	public:
		Compensate(PClip _child, Constant::Neighbour _source, MotionClip _motion, Engines::Engine *_luma, Engines::Engine *_chroma, IScriptEnvironment *env);
		PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment *env);
	};
	AVSValue __cdecl Create_SimpleCompensate(AVSValue args, void *user_data, IScriptEnvironment *env);
	AVSValue __cdecl Create_SimpleWarpCompensate(AVSValue args, void *user_data, IScriptEnvironment *env);


	class FPS : public GenericVideoFilter 
	{   
		Constant::Neighbour source;
		MotionClip motion;
		double ratio;
		Engines::Engine *luma, *chroma;

		__int64 totaltime;
		int nocalls;
	public:
		FPS(PClip _child, Constant::Neighbour _source, MotionClip _motion, double _newFPS, Engines::Engine *_luma, Engines::Engine *_chroma, IScriptEnvironment *env);
		PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment *env);
	};
	AVSValue __cdecl Create_SimpleFPS(AVSValue args, void *user_data, IScriptEnvironment *env);
	AVSValue __cdecl Create_SimpleWarpFPS(AVSValue args, void *user_data, IScriptEnvironment *env);

	class CombineFPS : public GenericVideoFilter 
	{   
		PClip fromprevious, fromnext;
		double ratio;
	public:
		CombineFPS(PClip _child, PClip _fromprevious, PClip _fromnext, double _newFPS, IScriptEnvironment *env);
		PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment *env);
	};
	AVSValue __cdecl Create_CombineFPS(AVSValue args, void *user_data, IScriptEnvironment *env);


	class BlendFPS : public GenericVideoFilter 
	{   
		double ratio;
		double aperture;
		int originalframecount;
	public:
		BlendFPS(PClip _child, double _newFPS, double _aperture, IScriptEnvironment *env);
		PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment *env);
	};
	AVSValue __cdecl Create_BlendFPS(AVSValue args, void *user_data, IScriptEnvironment *env);


	class SumStretchMask : public GenericVideoFilter 
	{
		int multiplier;
		MotionClip motion;
	public:
		SumStretchMask(PClip _child, MotionClip _motion, int _multiplier, IScriptEnvironment *env);
		PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment *env);
	};
	AVSValue __cdecl Create_SumStretchMask(AVSValue args, void *user_data, IScriptEnvironment *env);



	class SumStretchFPSMask : public GenericVideoFilter 
	{
		int multiplier;
		MotionClip motion;
		Constant::Neighbour source;
		double ratio;
	public:
		SumStretchFPSMask(PClip _child, MotionClip _motion, double _newFPS, Constant::Neighbour _source, int _multiplier, IScriptEnvironment *env);
		PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment *env);
	};
	AVSValue __cdecl Create_SumStretchFPSMask(AVSValue args, void *user_data, IScriptEnvironment *env);

}
