#pragma once
#include "Update.h"
#include "search.h"

namespace Primitives
{

	template<>
	inline int search<_Pixel>(const unsigned char *const currentposition, const int currentpitch, const unsigned char *const previousposition, const int previouspitch, SearchCandidate candidates[], int nocandidates)
	{
		int bestnumber = -1;
		int bestweight = 8*8*(256 + 4)*64;

		for(int i = 0; i < nocandidates; ++i)
		{
			int weight = candidates[i].penalty;
			//integer case only
			const unsigned char *currentscan = currentposition;
			const unsigned char *previousscan = previousposition + candidates[i].offset;
			for(int y = 0; y < 8; ++y, currentscan += currentpitch, previousscan += previouspitch)
				for(int x = 0; x < 8; ++x)
					weight += abs( currentscan[x] - previousscan[x] );

			if(weight < bestweight)
				bestnumber = i, bestweight = weight;
		}

		return bestnumber;
	}

	template<>
	inline int search_P3<_Pixel>(const unsigned char *const currentposition, const int currentpitch, const unsigned char *const previousposition, const int previouspitch, SearchCandidate candidates[], int nocandidates)
	{
	//++calls[ nocandidates];
		int localcurrentpitch = currentpitch;
		int localpreviouspitch = previouspitch;

		int bestnumber = -1;
		int bestweight = 8*8*(256 + 4)*64;
		for(int i = 0; i < nocandidates; ++i)
		{
			//integer case only
			const unsigned char *const previousblock = previousposition + candidates[i].offset;
			int result =0;
		
			__asm
			{
				mov	ecx, [localpreviouspitch]
				mov	edx, [localcurrentpitch]
					mov	esi, previousblock
				lea		eax, [ecx + 2*ecx]
				lea		ebx, [edx + 2*edx]

				//Load/compute the entire block being considered (previousblock)
					lea		edi, [esi + 4*ecx]
				movq	mm0, [esi]
				movq	mm1, [esi + ecx]
				movq	mm2, [esi + 2*ecx]
				movq	mm3, [esi + eax]
					mov	esi, currentposition
				movq	mm4, [edi]
				movq	mm5, [edi + ecx]
				movq	mm6, [edi + 2*ecx]
				movq	mm7, [edi + eax]

					lea edi, [esi + 4*edx]
				//compute SAD with the current block
				psadbw mm0, [esi]
				psadbw mm1, [esi + edx]
				psadbw mm2, [esi + 2*edx]
				psadbw mm3, [esi + ebx]
				psadbw mm4, [edi]
				psadbw mm5, [edi + edx]
				psadbw mm6, [edi + 2*edx]
				psadbw mm7, [edi + ebx]

				paddw	mm0, mm1
				paddw	mm2, mm3
				paddw	mm4, mm5
				paddw	mm6, mm7

				paddw	mm0, mm2
				paddw	mm4, mm6

				paddw	mm0, mm4

				movd eax, mm0
				mov	result, eax
			}
			result += candidates[i].penalty;

			if(result < bestweight)
				bestnumber = i, bestweight = result;

			//checking code
			//int referenceweight = candidates[i].penalty;
			//const unsigned char *currentscan = currentposition, *previousscan = previousblock;
			//for(int y = 0; y < 8; ++y, currentscan += currentpitch, previousscan += previouspitch)
			//	for(int x = 0; x < 8; ++x)
			//		referenceweight += min(abs( int(currentscan[x]) - previousscan[x] ), 255);
			//if(result!= referenceweight)	throw;
		}

		return bestnumber;
	}

}