#pragma once

namespace Primitives
{
	//because partial specialisation of function templates is not (yet) supported,
	//we need to mimic the effect

	template<int width, int height, class Interpolation>
	struct warpblock
	{
		warpblock(unsigned char *const to, int topitch, const unsigned char *const from, int frompitch, 
						const MotionVector topleftmotion, const MotionVector toprightmotion, 
						const MotionVector bottomleftmotion, const MotionVector bottomrightmotion);
	};
	template<int width, int height>
	struct warpblock<width, height, _Pixel>
	{
		warpblock(unsigned char *const to, int topitch, const unsigned char *const from, int frompitch, 
						const MotionVector topleftmotion, const MotionVector toprightmotion, 
						const MotionVector bottomleftmotion, const MotionVector bottomrightmotion);
	};
	template<int width, int height>
	struct warpblock<width, height, Bilinear>
	{
		warpblock(unsigned char *const to, int topitch, const unsigned char *const from, int frompitch, 
						const MotionVector topleftmotion, const MotionVector toprightmotion, 
						const MotionVector bottomleftmotion, const MotionVector bottomrightmotion);
	};


	template<int width, int height>
	warpblock<width, height, _Pixel>::warpblock(unsigned char *const to, int topitch, const unsigned char *const from, int frompitch, 
						const MotionVector topleftmotion, const MotionVector toprightmotion, 
						const MotionVector bottomleftmotion, const MotionVector bottomrightmotion)
	{
		//MotionVector topleftmotion = MotionVector (motion[0]);
		//MotionVector toprightmotion = MotionVector (motion[1]);
		//MotionVector bottomleftmotion = MotionVector (motion[motionpitch]);
		//MotionVector bottomrightmotion = MotionVector (motion[motionpitch + 1]);
		//The fractions used for interpolation are 1/16, 3/16, ... 15/16 (for width = height = 8)
		//Let the motion vector be (h, v)
		int dhdx = (toprightmotion.x		- topleftmotion.x)/width;
		int dvdx = (toprightmotion.y		- topleftmotion.y)/width;

		const int finaldhdx = (bottomrightmotion.x		- bottomleftmotion.x)/width;
		const int finaldvdx = (bottomrightmotion.y		- bottomleftmotion.y)/width;

		const int d2hdxdy = (finaldhdx		- dhdx)/height;
		const int d2vdxdy = (finaldvdx		- dvdx)/height;

		//left is 1/16 of a block in (for width = 8)
		const int dhleftdy = (bottomleftmotion.x	- topleftmotion.x)/height + d2hdxdy/2;
		const int dvleftdy = (bottomleftmotion.y	- topleftmotion.y)/height + d2vdxdy/2;

		//Obtain initial positions by moving right 1/16 of block and then down 1/16 of block (for width = 8)
		int hleft = topleftmotion.x + (dhdx + dhleftdy)/2;
		int vleft = topleftmotion.y + (dvdx + dvleftdy)/2;

		//Update horizontal increment to be 1/16 of a block down
		dhdx += d2hdxdy/2;
		dvdx += d2vdxdy/2;

		unsigned char *destinationscan = to;
		for(int y = 0; y < height; ++y,	hleft+=dhleftdy, vleft += dvleftdy, dhdx += d2hdxdy, dvdx += d2vdxdy, destinationscan += topitch)
		{
			int h = hleft;
			int v = vleft;
			for(int x = 0; x < width; ++x, h+=dhdx, v += dvdx)
			{
				int fromx = x+roundedpart(h);
				int fromy = y+roundedpart(v);
				const unsigned char *fromposition = from + fromy*frompitch + fromx;
				destinationscan[x]=*fromposition;
			}
		}
	}

	template<int width, int height>
	inline warpblock<width, height, class Bilinear>::warpblock(unsigned char *const to, int topitch, const unsigned char *const from, int frompitch,
						const MotionVector topleftmotion, const MotionVector toprightmotion, 
						const MotionVector bottomleftmotion, const MotionVector bottomrightmotion)
	{
		//MotionVector topleftmotion = MotionVector (motion[0]);
		//MotionVector toprightmotion = MotionVector (motion[1]);
		//MotionVector bottomleftmotion = MotionVector (motion[motionpitch]);
		//MotionVector bottomrightmotion = MotionVector (motion[motionpitch + 1]);
		//The fractions used for interpolation are 1/16, 3/16, ... 15/16 (for width = height = 8)
		//Let the motion vector be (h, v)
		int dhdx = (toprightmotion.x		- topleftmotion.x)/width;
		int dvdx = (toprightmotion.y		- topleftmotion.y)/width;

		const int finaldhdx = (bottomrightmotion.x		- bottomleftmotion.x)/width;
		const int finaldvdx = (bottomrightmotion.y		- bottomleftmotion.y)/width;

		const int d2hdxdy = (finaldhdx		- dhdx)/height;
		const int d2vdxdy = (finaldvdx		- dvdx)/height;

		//left is 1/16 of a block in (for width = 8)
		const int dhleftdy = (bottomleftmotion.x	- topleftmotion.x)/height + d2hdxdy/2;
		const int dvleftdy = (bottomleftmotion.y	- topleftmotion.y)/height + d2vdxdy/2;

		//Obtain initial positions by moving right 1/16 of block and then down 1/16 of block (for width = 8)
		int hleft = topleftmotion.x + (dhdx + dhleftdy)/2;
		int vleft = topleftmotion.y + (dvdx + dvleftdy)/2;

		//Update horizontal increment to be 1/16 of a block down
		dhdx += d2hdxdy/2;
		dvdx += d2vdxdy/2;

		unsigned char *destinationscan = to;
		for(int y = 0; y < height; ++y,	hleft+=dhleftdy, vleft += dvleftdy, dhdx += d2hdxdy, dvdx += d2vdxdy, destinationscan += topitch)
		{
			int h = hleft;
			int v = vleft;
			for(int x = 0; x < width; ++x, h+=dhdx, v += dvdx)
			{
				Fractional fractional( h,v );
				int fromx = x+		integralpart(h);
				int fromy = y+	integralpart(v);
				const unsigned char *toprow = from + fromy*frompitch + fromx;
				const unsigned char *bottomrow = toprow + frompitch;
				int topvalue = toprow[0] + (
					fractional.x()  ?  roundedpart(fractional.x()*(int(toprow[1])-toprow[0])) : 0
					);
				int bottomvalue = bottomrow[0] + (
					fractional.x()  ?  roundedpart(fractional.x()*(int(bottomrow[1])-bottomrow[0])) : 0
					);
				int result = topvalue + (
					fractional.y()	? roundedpart(fractional.y()*(bottomvalue - topvalue)) : 0
					);
				destinationscan[x] = static_cast<unsigned char>(result);
			}
		}
	}

}